@@ -24,6 +24,8 @@ class cryfs : public engines::engine
 public:
 	cryfs() ;
 
+	engine::engine::status passAllRequirenments( const engines::engine::options& opt ) const override ;
+
 	bool takesTooLongToUnlock() const override ;
 
 	const QProcessEnvironment& getProcessEnvironment() const override ;
@@ -33,10 +35,10 @@ public:
 	engines::engine::args command( const QByteArray& password,
 				       const engines::engine::cmdArgsList& args ) const override ;
 
-	const QString& installedVersionString() const override ;
-
 	void GUICreateOptionsinstance( QWidget * parent,engines::engine::function ) const override ;
 private:	
-	QProcessEnvironment m_env ;
-	engines::version m_version ;
+	QProcessEnvironment setEnv() const ;
+	const QProcessEnvironment m_env ;
+	const engines::versionGreaterOrEqual m_version_greater_or_equal_0_10_0 ;
+	const engines::versionGreaterOrEqual m_use_error_codes ;
 } ;
