@@ -31,7 +31,7 @@ public:
 
 	QStringList mountInfo( const QStringList& ) const override ;
 
-	const engines::engine& proveEngine( const QString& cipherPath ) const override ;
+	bool ownsCipherPath( const QString& cipherPath ) const override ;
 
 	void updateVolumeList( const engines::engine::options& ) const override ;
 
@@ -43,8 +43,6 @@ public:
 	engines::engine::args command( const QByteArray& password,
 				       const engines::engine::cmdArgsList& args ) const override ;
 
-	const QString& installedVersionString() const override ;
-
 	void GUICreateOptionsinstance( QWidget * parent,engines::engine::function ) const override ;
 private:
 	class unlockedVolumeList{
@@ -60,6 +58,6 @@ private:
 	} mutable m_unlockedVolumeManager ;
 
 	QString userOption() const ;
-	engines::exeFullPath m_exeFullPath ;
-	engines::version m_version ;
+
+	engines::versionGreaterOrEqual m_versionGreatorOrEqual_0_2_6 ;
 } ;
