@@ -57,7 +57,7 @@ class sirikali : public QWidget
 {
 	Q_OBJECT
 public:
-	sirikali() ;
+	sirikali( const QStringList& ) ;
 	int start( QApplication& ) ;
 	~sirikali() ;
 private slots:
@@ -70,7 +70,7 @@ private slots:
 	void polkitFailedWarning( void ) ;
 	void hideWindow( void ) ;
 	void setUpApp( const QString& ) ;
-	void start( const QStringList& ) ;
+	void start() ;
 	void autoUpdateCheck( void ) ;
 	void volumeProperties() ;
 	void genericVolumeProperties( void ) ;
@@ -96,6 +96,7 @@ private slots:
 	void slotCurrentItemChanged( QTableWidgetItem *,QTableWidgetItem * ) ;
 	void enableAll( void ) ;
 	void slotOpenFolder( void ) ;
+	void slotOpenParentFolder( void ) ;
 	void slotOpenSharedFolder( void ) ;
 	void addEntryToTable( const QStringList& ) ;
 	void addEntryToTable( const volumeInfo& ) ;
@@ -187,6 +188,8 @@ private:
 	debugWindow m_debugWindow ;
 
 	systemSignalHandler m_signalHandler ;
+
+	const QStringList& m_argumentList ;
 };
 
 #endif // MAINWINDOW_H
